package com.sun.imageio.plugins.jpeg;

import org.w3c.dom.NamedNodeMap;
import javax.imageio.stream.ImageOutputStream;
import javax.imageio.metadata.IIOMetadataNode;
import java.util.Iterator;
import org.w3c.dom.NodeList;
import javax.imageio.metadata.IIOInvalidTreeException;
import org.w3c.dom.Node;
import java.io.IOException;
import javax.imageio.plugins.jpeg.JPEGHuffmanTable;
import java.util.ArrayList;
import java.util.List;

class DHTMarkerSegment extends MarkerSegment
{
    List tables;
    
    DHTMarkerSegment(boolean b) {
        super(196);
        (this.tables = new ArrayList()).add(new DHTMarkerSegment.Htable(JPEGHuffmanTable.StdDCLuminance, true, 0));
        if (b) {
            this.tables.add(new DHTMarkerSegment.Htable(JPEGHuffmanTable.StdDCChrominance, true, 1));
        }
        this.tables.add(new DHTMarkerSegment.Htable(JPEGHuffmanTable.StdACLuminance, false, 0));
        if (b) {
            this.tables.add(new DHTMarkerSegment.Htable(JPEGHuffmanTable.StdACChrominance, false, 1));
        }
    }
    
    DHTMarkerSegment(JPEGBuffer jpegBuffer) throws IOException {
        super(jpegBuffer);
        this.tables = new ArrayList();
        Object o;
        for (int i = this.length; i > 0; i -= 17 + o.values.length) {
            o = new DHTMarkerSegment.Htable(jpegBuffer);
            this.tables.add(o);
        }
        jpegBuffer.bufAvail -= this.length;
    }
    
    DHTMarkerSegment(JPEGHuffmanTable[] array, JPEGHuffmanTable[] array2) {
        super(196);
        this.tables = new ArrayList();
        for (int i = 0; i < array.length; ++i) {
            this.tables.add(new DHTMarkerSegment.Htable(array[i], true, i));
        }
        for (int j = 0; j < array2.length; ++j) {
            this.tables.add(new DHTMarkerSegment.Htable(array2[j], false, j));
        }
    }
    
    DHTMarkerSegment(Node offendingNode) throws IIOInvalidTreeException {
        super(196);
        this.tables = new ArrayList();
        NodeList childNodes = offendingNode.getChildNodes();
        int length = childNodes.getLength();
        if (length < 1 || length > 4) {
            throw new IIOInvalidTreeException("Invalid DHT node", offendingNode);
        }
        for (int i = 0; i < length; ++i) {
            this.tables.add(new DHTMarkerSegment.Htable(childNodes.item(i)));
        }
        return;
    }
    
    @Override
    protected Object clone() {
        DHTMarkerSegment dhtMarkerSegment = (DHTMarkerSegment)super.clone();
        dhtMarkerSegment.tables = new ArrayList(this.tables.size());
        Iterator<Object> iterator = this.tables.iterator();
        while (iterator.hasNext()) {
            dhtMarkerSegment.tables.add(((DHTMarkerSegment.Htable)iterator.next()).clone());
        }
        return dhtMarkerSegment;
    }
    
    @Override
    IIOMetadataNode getNativeNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("dht");
        for (int i = 0; i < this.tables.size(); ++i) {
            iioMetadataNode.appendChild(((DHTMarkerSegment.Htable)this.tables.get(i)).getNativeNode());
        }
        return iioMetadataNode;
    }
    
    @Override
    void write(ImageOutputStream imageOutputStream) throws IOException {
    }
    
    @Override
    void print() {
        this.printTag("DHT");
        System.out.println("Num tables: " + Integer.toString(this.tables.size()));
        for (int i = 0; i < this.tables.size(); ++i) {
            ((DHTMarkerSegment.Htable)this.tables.get(i)).print();
        }
        System.out.println();
    }
    
    DHTMarkerSegment.Htable getHtableFromNode(Node node) throws IIOInvalidTreeException {
        return new DHTMarkerSegment.Htable(node);
    }
    
    void addHtable(JPEGHuffmanTable jpegHuffmanTable, boolean b, int n) {
        this.tables.add(new DHTMarkerSegment.Htable(jpegHuffmanTable, b, n));
    }
    
    class Htable implements Cloneable
    {
        int tableClass;
        int tableID;
        private static final int NUM_LENGTHS = 16;
        short[] numCodes;
        short[] values;
        
        Htable(JPEGBuffer jpegBuffer) {
            this.numCodes = new short[16];
            this.tableClass = jpegBuffer.buf[jpegBuffer.bufPtr] >>> 4;
            this.tableID = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xF);
            for (int i = 0; i < 16; ++i) {
                this.numCodes[i] = (short)(jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF);
            }
            int n = 0;
            for (int j = 0; j < 16; ++j) {
                n += this.numCodes[j];
            }
            this.values = new short[n];
            for (int k = 0; k < n; ++k) {
                this.values[k] = (short)(jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF);
            }
        }
        
        Htable(JPEGHuffmanTable jpegHuffmanTable, boolean b, int tableID) {
            this.numCodes = new short[16];
            this.tableClass = !b;
            this.tableID = tableID;
            this.numCodes = jpegHuffmanTable.getLengths();
            this.values = jpegHuffmanTable.getValues();
        }
        
        Htable(Node node) throws IIOInvalidTreeException {
            this.numCodes = new short[16];
            if (node.getNodeName().equals("dhtable")) {
                NamedNodeMap attributes = node.getAttributes();
                if (attributes.getLength() != 2) {
                    throw new IIOInvalidTreeException("dhtable node must have 2 attributes", node);
                }
                this.tableClass = MarkerSegment.getAttributeValue(node, attributes, "class", 0, 1, true);
                this.tableID = MarkerSegment.getAttributeValue(node, attributes, "htableId", 0, 3, true);
                if ((node instanceof IIOMetadataNode)) {
                    JPEGHuffmanTable jpegHuffmanTable = (JPEGHuffmanTable)((IIOMetadataNode)node).getUserObject();
                    if (jpegHuffmanTable == null) {
                        throw new IIOInvalidTreeException("dhtable node must have user object", node);
                    }
                    this.numCodes = jpegHuffmanTable.getLengths();
                    this.values = jpegHuffmanTable.getValues();
                    return;
                }
                else {
                    throw new IIOInvalidTreeException("dhtable node must have user object", node);
                }
            }
            else {
                throw new IIOInvalidTreeException("Invalid node, expected dqtable", node);
            }
        }
        
        @Override
        protected Object clone() {
            DHTMarkerSegment.Htable htable = null;
            try {
                htable = (DHTMarkerSegment.Htable)super.clone();
            }
            catch (CloneNotSupportedException ex) {}
            if (this.numCodes != null) {
                htable.numCodes = (short[])(short[])this.numCodes.clone();
            }
            if (this.values != null) {
                htable.values = (short[])(short[])this.values.clone();
                return htable;
            }
            return htable;
        }
        
        IIOMetadataNode getNativeNode() {
            IIOMetadataNode iioMetadataNode = new IIOMetadataNode("dhtable");
            iioMetadataNode.setAttribute("class", Integer.toString(this.tableClass));
            iioMetadataNode.setAttribute("htableId", Integer.toString(this.tableID));
            iioMetadataNode.setUserObject(new JPEGHuffmanTable(this.numCodes, this.values));
            return iioMetadataNode;
        }
        
        void print() {
            System.out.println("Huffman Table");
            System.out.println("table class: " + ((this.tableClass == 0) ? "DC" : "AC"));
            System.out.println("table id: " + Integer.toString(this.tableID));
            new JPEGHuffmanTable(this.numCodes, this.values).toString();
        }
    }
}
